package apis

import (
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/go-admin-team/go-admin-core/sdk/api"
	"github.com/go-admin-team/go-admin-core/sdk/pkg/jwtauth/user"
	_ "github.com/go-admin-team/go-admin-core/sdk/pkg/response"

	"go-admin/app/mobile/service"
	"go-admin/app/mobile/service/dto"
	"go-admin/app/operate/models"
	"go-admin/common/actions"
)

type OrgPlayerUser struct {
	api.Api
}

// GetPage 获取球员账户信息列表
// @Summary 获取球员账户信息列表
// @Description 获取球员账户信息列表
// @Tags 球员账户信息
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Success 200 {string} string {"code": 200, "data": [...]}
// @Router /mobile/v1/org-player-user/login [post]
// @Security Bearer
func (e OrgPlayerUser) Login(c *gin.Context) {
	req := dto.LoginReq{}
	s := service.OrgPlayerUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	p := actions.GetPermissionFromContext(c)
	data := new(models.OrgPlayerUser)
	req.Password = dto.Pwd(req.Password)

	err = s.Login(&req, p, data)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取球员账户信息 失败，\r\n失败信息 %s", err.Error()))
		return
	}

	//自建token生成
	reply := new(dto.LoginReply)
	//token, _ := utils.GenToken(int64(data.Id))
	//
	//a,_ := jwt.New(&jwt.GinJWTMiddleware{
	//	Realm:           "test zone",
	//	Key:             []byte(config.JwtConfig.Secret),
	//	Timeout:         time.Duration(876010) * time.Hour,
	//	MaxRefresh:      time.Hour,
	//	PayloadFunc:     handler.PayloadFunc,
	//	IdentityHandler: handler.IdentityHandler,
	//	Authenticator:   handler.Authenticator,
	//	Authorizator:    handler.Authorizator,
	//	Unauthorized:    handler.Unauthorized,
	//	TokenLookup:     "header: Authorization, query: token, cookie: jwt",
	//	TokenHeadName:   "Bearer",
	//	TimeFunc:        time.Now,
	//})
	//reply.Token = a.
	//reply.PlayerUserId = strconv.Itoa(data.Id)

	e.OK(reply, "查询成功")
}

// GetPage 获取球员账户信息列表
// @Summary 获取球员账户信息列表
// @Description 获取球员账户信息列表
// @Tags 球员账户信息
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Success 200 {string} string {"code": 200, "data": [...]}
// @Router /api/v1/org-player-user [get]
// @Security Bearer
func (e OrgPlayerUser) GetPage(c *gin.Context) {
	req := dto.OrgPlayerUserGetPageReq{}
	s := service.OrgPlayerUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	p := actions.GetPermissionFromContext(c)
	list := make([]models.OrgPlayerUser, 0)
	var count int64

	err = s.GetPage(&req, p, &list, &count)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取球员账户信息 失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 获取球员账户信息
// @Summary 获取球员账户信息
// @Description 获取球员账户信息
// @Tags 球员账户信息
// @Param id path string false "id"
// @Success 200 {string} string "{"code": 200,"msg":"更新成功","data":{}}"
// @Router /api/v1/org-player-user/{id} [get]
// @Security Bearer
func (e OrgPlayerUser) Get(c *gin.Context) {
	req := dto.OrgPlayerUserGetReq{}
	s := service.OrgPlayerUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object models.OrgPlayerUser

	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, p, &object)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取球员账户信息失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(object, "查询成功")
}

// Insert 创建球员账户信息
// @Summary 创建球员账户信息
// @Description 创建球员账户信息
// @Tags 球员账户信息
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgPlayerUserInsertReq true "data"
// @Success 200 {string} string  	"{"code": 200, "message": "添加成功"}"
// @Router /api/v1/org-player-user [post]
// @Security Bearer
func (e OrgPlayerUser) Insert(c *gin.Context) {
	req := dto.OrgPlayerUserInsertReq{}
	s := service.OrgPlayerUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))

	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("创建球员账户信息  失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(req.GetId(), "创建成功")
}

// Update 修改球员账户信息
// @Summary 修改球员账户信息
// @Description 修改球员账户信息
// @Tags 球员账户信息
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgPlayerUserUpdateReq true "body"
// @Success 200 {string} string  	"{"code": 200, "message": "修改成功"}"
// @Router /api/v1/org-player-user/{id} [put]
// @Security Bearer
func (e OrgPlayerUser) Update(c *gin.Context) {
	req := dto.OrgPlayerUserUpdateReq{}
	s := service.OrgPlayerUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("修改球员账户信息 失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "修改成功")
}

// Delete 删除球员账户信息
// @Summary 删除球员账户信息
// @Description 删除球员账户信息
// @Tags 球员账户信息
// @Param ids body []int false "ids"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Router /api/v1/org-player-user [delete]
// @Security Bearer
func (e OrgPlayerUser) Delete(c *gin.Context) {
	s := service.OrgPlayerUser{}
	req := dto.OrgPlayerUserDeleteReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("删除球员账户信息失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "删除成功")
}
