package apis

import (
	"fmt"

	"github.com/gin-gonic/gin"
	"github.com/go-admin-team/go-admin-core/sdk/api"
	"github.com/go-admin-team/go-admin-core/sdk/pkg/jwtauth/user"
	_ "github.com/go-admin-team/go-admin-core/sdk/pkg/response"

	"go-admin/app/operate/models"
	"go-admin/app/operate/service"
	"go-admin/app/operate/service/dto"
	"go-admin/common/actions"
)

type OrgAd struct {
	api.Api
}

// GetPage <运营>获取广告列表
// @Summary <运营>获取广告列表
// @Description <运营>获取广告列表
// @Tags <运营>广告
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Param data body dto.OrgAdGetPageReq true "data"
// @Success 200 {string} string "{"code": 200, "data": [...]}"
// @Router /api/v1/org-ad [get]
// @Security Bearer
func (e OrgAd) GetPage(c *gin.Context) {
	req := dto.OrgAdGetPageReq{}
	s := service.OrgAd{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	p := actions.GetPermissionFromContext(c)
	list := make([]models.OrgAd, 0)
	var count int64

	err = s.GetPage(&req, p, &list, &count)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取广告 失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get <运营>获取广告
// @Summary <运营>获取广告
// @Description <运营>获取广告
// @Tags <运营>广告
// @Param id path string false "id"
// @Success 200 {string} string  {data=models.OrgAd} "{"code": 200, "data": [...]}"
// @Router /api/v1/org-ad/{id} [get]
// @Security Bearer
func (e OrgAd) Get(c *gin.Context) {
	req := dto.OrgAdGetReq{}
	s := service.OrgAd{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object models.OrgAd

	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, p, &object)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取广告失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(object, "查询成功")
}

// Insert <运营>创建广告
// @Summary <运营>创建广告
// @Description <运营>创建广告
// @Tags <运营>广告
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgAdInsertReq true "data"
// @Success 200 {string} string  	"{"code": 200, "message": "添加成功"}"
// @Router /api/v1/org-ad [post]
// @Security Bearer
func (e OrgAd) Insert(c *gin.Context) {
	req := dto.OrgAdInsertReq{}
	s := service.OrgAd{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))

	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("创建广告  失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(req.GetId(), "创建成功")
}

// Update <运营>修改广告
// @Summary <运营>修改广告
// @Description <运营>修改广告
// @Tags <运营>广告
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgAdUpdateReq true "body"
// @Success 200 {string} string  	"{"code": 200, "message": "修改成功"}"
// @Router /api/v1/org-ad/{id} [put]
// @Security Bearer
func (e OrgAd) Update(c *gin.Context) {
	req := dto.OrgAdUpdateReq{}
	s := service.OrgAd{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("修改广告 失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "修改成功")
}

// Delete <运营>删除广告
// @Summary <运营>删除广告
// @Description <运营>删除广告
// @Tags <运营>广告
// @Param ids body []int false "ids"
// @Success 200 {string} string  	"{"code": 200, "message": "删除成功"}"
// @Router /api/v1/org-ad [delete]
// @Security Bearer
func (e OrgAd) Delete(c *gin.Context) {
	s := service.OrgAd{}
	req := dto.OrgAdDeleteReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("删除广告失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "删除成功")
}
