package apis

import (
	"fmt"

	"github.com/gin-gonic/gin"
	"github.com/go-admin-team/go-admin-core/sdk/api"
	"github.com/go-admin-team/go-admin-core/sdk/pkg/jwtauth/user"
	_ "github.com/go-admin-team/go-admin-core/sdk/pkg/response"

	"go-admin/app/operate/models"
	"go-admin/app/operate/service"
	"go-admin/app/operate/service/dto"
	"go-admin/common/actions"
)

type OrgMatchTeamPlayer struct {
	api.Api
}

// GetPage 获取比赛球员数据列表
// @Summary 获取比赛球员数据列表
// @Description 获取比赛球员数据列表
// @Tags 比赛球员数据
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Param data body dto.OrgMatchTeamPlayerGetPageReq true "data"
// @Success 200 {string} string  "{"code": 200, "data": [...]}"
// @Router /api/v1/org-match-team-player [get]
// @Security Bearer
func (e OrgMatchTeamPlayer) GetPage(c *gin.Context) {
	req := dto.OrgMatchTeamPlayerGetPageReq{}
	s := service.OrgMatchTeamPlayer{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	p := actions.GetPermissionFromContext(c)
	list := make([]models.OrgMatchTeamPlayer, 0)
	var count int64

	err = s.GetPage(&req, p, &list, &count)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取比赛球员数据 失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 获取比赛球员数据
// @Summary 获取比赛球员数据
// @Description 获取比赛球员数据
// @Tags 比赛球员数据
// @Param id path string false "id"
// @Success 200 {string} string  "{"code": 200, "data": [...]}"
// @Router /api/v1/org-match-team-player/{id} [get]
// @Security Bearer
func (e OrgMatchTeamPlayer) Get(c *gin.Context) {
	req := dto.OrgMatchTeamPlayerGetReq{}
	s := service.OrgMatchTeamPlayer{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object models.OrgMatchTeamPlayer

	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, p, &object)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取比赛球员数据失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(object, "查询成功")
}

// Insert 创建比赛球员数据
// @Summary 创建比赛球员数据
// @Description 创建比赛球员数据
// @Tags 比赛球员数据
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgMatchTeamPlayerInsertReq true "data"
// @Success 200 {string} string  	"{"code": 200, "message": "添加成功"}"
// @Router /api/v1/org-match-team-player [post]
// @Security Bearer
func (e OrgMatchTeamPlayer) Insert(c *gin.Context) {
	req := dto.OrgMatchTeamPlayerInsertReq{}
	s := service.OrgMatchTeamPlayer{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))

	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("创建比赛球员数据  失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(req.GetId(), "创建成功")
}

// Update 修改比赛球员数据
// @Summary 修改比赛球员数据
// @Description 修改比赛球员数据
// @Tags 比赛球员数据
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgMatchTeamPlayerUpdateReq true "body"
// @Success 200 {string} string  	"{"code": 200, "message": "修改成功"}"
// @Router /api/v1/org-match-team-player/{id} [put]
// @Security Bearer
func (e OrgMatchTeamPlayer) Update(c *gin.Context) {
	req := dto.OrgMatchTeamPlayerUpdateReq{}
	s := service.OrgMatchTeamPlayer{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("修改比赛球员数据 失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "修改成功")
}

// Delete 删除比赛球员数据
// @Summary 删除比赛球员数据
// @Description 删除比赛球员数据
// @Tags 比赛球员数据
// @Param ids body []int false "ids"
// @Success 200 {string} string  	"{"code": 200, "message": "删除成功"}"
// @Router /api/v1/org-match-team-player [delete]
// @Security Bearer
func (e OrgMatchTeamPlayer) Delete(c *gin.Context) {
	s := service.OrgMatchTeamPlayer{}
	req := dto.OrgMatchTeamPlayerDeleteReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("删除比赛球员数据失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "删除成功")
}
