package apis

import (
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/go-admin-team/go-admin-core/sdk/api"
	"github.com/go-admin-team/go-admin-core/sdk/pkg/jwtauth/user"
	_ "github.com/go-admin-team/go-admin-core/sdk/pkg/response"

	"go-admin/app/operate/models"
	"go-admin/app/operate/service"
	"go-admin/app/operate/service/dto"
	"go-admin/common/actions"
)

type OrgSeason struct {
	api.Api
}

// GetPage <赛事>获取赛季列表
// @Summary <赛事>获取赛季列表
// @Description <赛事>获取赛季列表
// @Tags <赛事>赛季
// @Param pageSize query int false "页条数"
// @Param pageIndex query int false "页码"
// @Param data body dto.OrgSeasonGetPageReq true "data"
// @Success 200 {string} string  {data=response.Page{list=[]models.OrgSeason}} "{"code": 200, "data": [...]}"
// @Router /api/v1/org-season [get]
// @Security Bearer
func (e OrgSeason) GetPage(c *gin.Context) {
	req := dto.OrgSeasonGetPageReq{}
	s := service.OrgSeason{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	p := actions.GetPermissionFromContext(c)
	list := make([]dto.OrgSeasonGetPageReply, 0)
	var count int64

	err = s.GetPage(&req, p, &list, &count)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取赛季 失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get <赛事>获取赛季
// @Summary <赛事>获取赛季
// @Description <赛事>获取赛季
// @Tags <赛事>赛季
// @Param id path string false "id"
// @Success 200 {string} string  {data=models.OrgSeason} "{"code": 200, "data": [...]}"
// @Router /api/v1/org-season/{id} [get]
// @Security Bearer
func (e OrgSeason) Get(c *gin.Context) {
	req := dto.OrgSeasonGetReq{}
	s := service.OrgSeason{}
	err := e.MakeContext(c).MakeOrm().Bind(&req).MakeService(&s.Service).Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object models.OrgSeason

	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, p, &object)
	if err != nil {
		//e.Error(500, err, fmt.Sprintf("获取赛季失败，\r\n失败信息 %s", err.Error()))
		//return
	}

	//联赛级别列表
	sOrgLeague := service.OrgLeague{}
	listOrgLeague := make([]models.OrgLeague, 0)
	var countOrgLeague int64 = 0

	err = e.MakeContext(c).MakeOrm().MakeService(&sOrgLeague.Service).Errors

	reqOrgLeague := new(dto.OrgLeagueGetPageReq)
	reqOrgLeague.PageIndex = 1
	reqOrgLeague.PageSize = 999

	err = sOrgLeague.GetPage(reqOrgLeague, p, &listOrgLeague, &countOrgLeague)

	if err != nil {
		e.Error(500, err, fmt.Sprintf("获取联赛 失败，\r\n失败信息 %s", err.Error()))
		return
	}

	reply := dto.OrgSeasonGetReply{OrgLeagueList: listOrgLeague, OrgSeason: object}

	e.OK(reply, "查询成功")
}

// Insert <赛事>创建赛季
// @Summary <赛事>创建赛季
// @Description <赛事>创建赛季
// @Tags <赛事>赛季
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgSeasonInsertReq true "data"
// @Success 200 {string} string  	"{"code": 200, "message": "添加成功"}"
// @Router /api/v1/org-season [post]
// @Security Bearer
func (e OrgSeason) Insert(c *gin.Context) {
	req := dto.OrgSeasonInsertReq{}
	s := service.OrgSeason{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))

	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("创建赛季  失败，\r\n失败信息 %s", err.Error()))
		return
	}

	e.OK(req.GetId(), "创建成功")
}

// Update <赛事>修改赛季
// @Summary <赛事>修改赛季
// @Description <赛事>修改赛季
// @Tags <赛事>赛季
// @Accept application/json
// @Product application/json
// @Param data body dto.OrgSeasonUpdateReq true "body"
// @Success 200 {string} string  	"{"code": 200, "message": "修改成功"}"
// @Router /api/v1/org-season/{id} [put]
// @Security Bearer
func (e OrgSeason) Update(c *gin.Context) {
	req := dto.OrgSeasonUpdateReq{}
	s := service.OrgSeason{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("修改赛季 失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "修改成功")
}

// Delete 删除赛季
// @Summary 删除赛季
// @Description 删除赛季
// @Tags <赛事>赛季
// @Param ids body []int false "ids"
// @Success 200 {string} string  	"{"code": 200, "message": "删除成功"}"
// @Router /api/v1/org-season [delete]
// @Security Bearer
func (e OrgSeason) Delete(c *gin.Context) {
	s := service.OrgSeason{}
	req := dto.OrgSeasonDeleteReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// req.SetUpdateBy(user.GetUserId(c))
	p := actions.GetPermissionFromContext(c)

	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, fmt.Sprintf("删除赛季失败，\r\n失败信息 %s", err.Error()))
		return
	}
	e.OK(req.GetId(), "删除成功")
}
