package service

import (
	"errors"

	"github.com/go-admin-team/go-admin-core/sdk/service"
	"gorm.io/gorm"

	"go-admin/app/operate/models"
	"go-admin/app/operate/service/dto"
	"go-admin/common/actions"
	cDto "go-admin/common/dto"
)

type OrgMatch struct {
	service.Service
}

// GetPage 获取OrgMatch列表
func (e *OrgMatch) GetPage(c *dto.OrgMatchGetPageReq, p *actions.DataPermission, list *[]dto.OrgMatchGetPageReply, count *int64) error {
	var err error
	var data models.OrgMatch

	err = e.Orm.Table("org_match as om").
		Select("om.id, ol.league_name, od.division_name, os.season_name,om.grouping,"+
			"om.rounds, om.match_start_time, om.match_end_time, om.status,"+
			"ota.team_name as team_a_name, om.team_a_id,om.team_b_id, otb.team_name as team_b_name").
		Joins("left join org_league as ol on ol.id = om.league_id").
		Joins("left join org_division as od on od.id = om.division_id").
		Joins("left join org_season as os on os.id = om.season_id").
		Joins("left join org_team as ota on ota.id = om.team_a_id").
		Joins("left join org_team as otb on otb.id = om.team_b_id").
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			actions.Permission(data.TableName(), p),
			cDto.PassDel("om"),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("OrgMatchService GetPage error:%s \r\n", err)
		return err
	}
	return nil
}

// GetPage 获取OrgMatch列表
func (e *OrgMatch) GetPageForTeam(c *dto.OrgMatchGetPageReq, p *actions.DataPermission, list *[]models.OrgMatchTeamPlayer, count *int64) error {
	var err error
	var data models.OrgMatch

	err = e.Orm.Table("org_match as om").
		Select("om.id, ol.league_name, od.division_name, os.season_name,om.grouping,"+
			"om.rounds, om.match_start_time, om.match_end_time, om.status,"+
			"ota.team_name as team_a_name, otb.team_name as team_b_name").
		Joins("left join org_league as ol on ol.id = om.league_id").
		Joins("left join org_division as od on od.id = om.division_id").
		Joins("left join org_season as os on os.id = om.season_id").
		Joins("left join org_team as ota on ota.id = om.team_a_id").
		Joins("left join org_team as otb on otb.id = om.team_b_id").
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			cDto.PassDel("om"),
			actions.Permission(data.TableName(), p),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("OrgMatchService GetPage error:%s \r\n", err)
		return err
	}
	return nil
}

// Get 获取OrgMatch对象
func (e *OrgMatch) Get(id string, p *actions.DataPermission, model *models.OrgMatch) error {
	var data models.OrgMatch

	err := e.Orm.Model(&data).
		Scopes(
			actions.Permission(data.TableName(), p),
		).
		First(model, id).Error
	if err != nil && errors.Is(err, gorm.ErrRecordNotFound) {
		err = errors.New("查看对象不存在或无权查看")
		e.Log.Errorf("Service GetOrgMatch error:%s \r\n", err)
		return err
	}
	if err != nil {
		e.Log.Errorf("db error:%s", err)
		return err
	}
	return nil
}

// Insert 创建OrgMatch对象
func (e *OrgMatch) Insert(c *dto.OrgMatchInsertReq) error {
	var err error
	var data models.OrgMatch
	c.Generate(&data)
	err = e.Orm.Create(&data).Error
	if err != nil {
		e.Log.Errorf("OrgMatchService Insert error:%s \r\n", err)
		return err
	}
	return nil
}

// Update 修改OrgMatch对象
func (e *OrgMatch) Update(c *dto.OrgMatchUpdateReq, p *actions.DataPermission) error {
	var err error
	var data = models.OrgMatch{}
	e.Orm.Scopes(
		actions.Permission(data.TableName(), p),
	).First(&data, c.GetId())
	c.Generate(&data)

	db := e.Orm.Updates(&data)
	if db.Error != nil {
		e.Log.Errorf("OrgMatchService Save error:%s \r\n", err)
		return err
	}
	if db.RowsAffected == 0 {
		return errors.New("无权更新该数据")
	}
	return nil
}

// Remove 删除OrgMatch
func (e *OrgMatch) Remove(d *dto.OrgMatchDeleteReq, p *actions.DataPermission) error {
	var data models.OrgMatch

	db := e.Orm.Model(&data).
		Scopes(
			actions.Permission(data.TableName(), p),
		).Delete(&data, d.GetId())
	if err := db.Error; err != nil {
		e.Log.Errorf("Service RemoveOrgMatch error:%s \r\n", err)
		return err
	}
	if db.RowsAffected == 0 {
		return errors.New("无权删除该数据")
	}
	return nil
}
